define([
    'jquery',
], function ($) {
    'use strict';

    $.widget('mage.qtyWidget', {
        options: {
            increment: 1,
            qtyInput: ".input-text.qty",
            productUpdateCartButton: "#product-updatecart-button",
            shoppingCartTable: "#shopping-cart-table",
            automaticUpdateCartEvent: false,
        },

        _create: function() {
            this.increment = this.options.increment;

            if($(this.options.qtyInput)) {
                // We disable the qyt manual input changes and validation in all the cases
                $(this.options.qtyInput).attr('readonly', true);
                $(this.options.qtyInput).removeAttr('data-validate');

                // We create the plus and minus buttons
                if($(this.options.qtyInput).prev('.qty-button').length === 0) {
                    $('<a type="button" class="qty-button qty-decrement" data-role="change-qty"><span>minus</span></a>').insertBefore(this.options.qtyInput);
                }
                if($(this.options.qtyInput).next('.qty-button').length === 0) {
                    $('<a type="button" class="qty-button qty-increment" data-role="change-qty"><span>plus</span></a>').insertAfter(this.options.qtyInput);
                }

                // If we detect the shoppingCartTable we add auto update functionality
                if($(this.options.shoppingCartTable).length !== 0 && $('.hide.tocart').length === 0) {
                    $('<button type="submit" class="hide tocart" id="product-updatecart-button"></button>').insertAfter(this.options.shoppingCartTable);
                    this.options.automaticUpdateCartEvent = true;
                }
            }

            this._bindEvents();
        },

        _bindEvents: function(){
            var _self = this,
                $decrement = $(this.element).parent().find('.qty-decrement'),
                $increment = $(this.element).parent().find('.qty-increment');

            $decrement.on('mouseup', function(e){
                e.preventDefault();
                _self._qtyDecrement(_self, this, e);
            });

            $decrement.on('keyup', function(e){
                if(e.keyCode === 13){
                    e.preventDefault();
                    _self._qtyDecrement(_self, this, e);
                }
            });

            $increment.on('mouseup', function(e){
                e.preventDefault();
                _self._qtyIncrement(_self, this, e);
            });

            $increment.on('keyup', function(e){
                if(e.keyCode === 13){
                    e.preventDefault();
                    _self._qtyIncrement(_self, this, e);
                }
            });
        },

        _qtyDecrement: function(self, target){
            this._adjustQty(self, target, -self.increment);
        },

        _qtyIncrement: function(self, target){
            this._adjustQty(self, target, self.increment);
        },

        _adjustQty: function(self, target, step) {
            var currentInput = $(target).parent().find('input'),
                currentQty = parseInt($(currentInput).val());

            if(isNaN(currentQty)) {
                currentQty = 1;
            }

            currentQty += step;

            if(currentQty < 1) {
                currentQty = 1;
            }

            $(currentInput).val(currentQty)
                .trigger('input');

            // If automaticUpdateCartEvent is set we call the AJAX addToCart method
            if($(this.options.shoppingCartTable).length !== 0 &&
                $(this.options.productUpdateCartButton).length !== 0) {
                $(this.options.productUpdateCartButton).trigger("click");
            }
        },
    });

    return $.mage.qtyWidget;
});
